using System.Collections.Generic;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;

namespace RH.UploadAndInsertField
{
    public class RHUploadAndInsert : SPFieldText
    {
        private static readonly string[] CustomPropertyNames = new[] { "DocumentLibraryName", "PictureLibraryName", "FieldToInsertLink" };

        public RHUploadAndInsert(SPFieldCollection fields, string fieldName)
            : base(fields, fieldName)
        {
            InitProperties();
        }

        public RHUploadAndInsert(SPFieldCollection fields, string typeName, string displayName)
            : base(fields, typeName, displayName)
        {
            InitProperties();
        }

        #region Property storage and bug workarounds - do not edit
        /// <summary>
        /// Indicates that the field is being created rather than edited. This is necessary to 
        /// work around some bugs in field creation.
        /// </summary>
        public bool IsNew { get; set; }

        /// <summary>
        /// Backing fields for custom properties. Using a dictionary to make it easier to abstract
        /// details of working around SharePoint bugs.
        /// </summary>
        private readonly Dictionary<string, string> _CustomProperties = new Dictionary<string, string>();

        /// <summary>
        /// Static store to transfer custom properties between instances. This is needed to allow
        /// correct saving of custom properties when a field is created - the custom property 
        /// implementation is not used by any out of box SharePoint features so is really buggy.
        /// </summary>
        private static readonly Dictionary<string, string> CustomPropertiesForNewFields = new Dictionary<string, string>();

        /// <summary>
        /// Initialise backing fields from base property store
        /// </summary>
        private void InitProperties()
        {
            foreach (string propertyName in CustomPropertyNames)
            {
                _CustomProperties[propertyName] = base.GetCustomProperty(propertyName) + "";
            }
        }

        /// <summary>
        /// Take properties from either the backing fields or the static store and 
        /// put them in the base property store
        /// </summary>
        private void SaveProperties()
        {
            foreach (string propertyName in CustomPropertyNames)
            {
                base.SetCustomProperty(propertyName, GetCustomProperty(propertyName));
            }
        }

        /// <summary>
        /// Get an identifier for the field being added/edited that will be unique even if
        /// another user is editing a property of the same name.
        /// </summary>
        /// <param name="propertyName"></param>
        /// <returns></returns>
        private string GetCacheKey(string propertyName)
        {
           string key = "_" + (ParentList == null ? "SITE" : ParentList.ID.ToString()) + "_" + propertyName;
           if (SPContext.Current == null)
           {
              return key;
           }
           return SPContext.Current.GetHashCode() + key;
        }

       /// <summary>
        /// Replace the buggy base implementation of SetCustomProperty
        /// </summary>
        /// <param name="propertyName"></param>
        /// <param name="propertyValue"></param>
        new public void SetCustomProperty(string propertyName, object propertyValue)
        {
            if (IsNew)
            {
                // field is being added - need to put property in cache
                CustomPropertiesForNewFields[GetCacheKey(propertyName)] = propertyValue + "";
            }

            _CustomProperties[propertyName] = propertyValue + "";
        }

        /// <summary>
        /// Replace the buggy base implementation of GetCustomProperty
        /// </summary>
        /// <param name="propertyName"></param>
        new public object GetCustomProperty(string propertyName)
        {
            if (!IsNew && CustomPropertiesForNewFields.ContainsKey(GetCacheKey(propertyName)))
            {
                string s = CustomPropertiesForNewFields[GetCacheKey(propertyName)];
                CustomPropertiesForNewFields.Remove(GetCacheKey(propertyName));
                _CustomProperties[propertyName] = s;
                return s;
            }
            return _CustomProperties[propertyName];
        }

        /// <summary>
        /// Called when a field is created. Without this, update is not called and custom properties
        /// are not saved.
        /// </summary>
        /// <param name="op"></param>
        public override void OnAdded(SPAddFieldOptions op)
        {
            base.OnAdded(op);
            Update();
        }

        #endregion


        public override BaseFieldControl FieldRenderingControl
        {
            get
            {
                BaseFieldControl fieldControl = new RHUploadAndInsertControl(this) {FieldName = InternalName};
                return fieldControl;
            }
        }

        public override void Update()
        {
            SaveProperties();
            base.Update();
        }

        public string DocumentLibraryName
        {
            get { return GetCustomProperty("DocumentLibraryName") + ""; }
            set { SetCustomProperty("DocumentLibraryName", value); }
        }

        public string PictureLibraryName
        {
            get { return GetCustomProperty("PictureLibraryName") + ""; }
            set { SetCustomProperty("PictureLibraryName", value); }
        }

        public string FieldToInsertLink
        {
            get { return GetCustomProperty("FieldToInsertLink") + ""; }
            set { SetCustomProperty("FieldToInsertLink", value); }
        }
    }
}
