using System;
using System.Web.UI;
using Microsoft.SharePoint.WebControls;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using System.Text;

namespace RH.UploadAndInsertField
{
	public class RHUploadAndInsertControl : BaseFieldControl
	{
		private readonly RHUploadAndInsert field;
		private FileUpload _FileUpload;
		private SPList _DocLib;
		private SPList _PicLib;
		private SPField _FieldToInsertLink;
		private RadioButtonList _LibChoose;

		public RHUploadAndInsertControl(RHUploadAndInsert parentField)
		{
			field = parentField;
		}

		protected override void CreateChildControls()
		{
			base.CreateChildControls();

			try
			{
				// get objects from field properties
				GetLibraries();
				// add controls
				string fieldName = _FieldToInsertLink != null ? _FieldToInsertLink.Title : string.Empty;
				string hint = string.Format(Helper.GetResource("labelSelectFile"), fieldName);
				Controls.Add(new Label { ID = "UploadHint", Text = hint });
				Controls.AddLinebreak(1);
				_FileUpload = new FileUpload { ID = "FileUpload" };
				Controls.Add(_FileUpload);
				Controls.Add(new LiteralControl("&nbsp;"));

				_LibChoose = new RadioButtonList
									  {
										  ID = "LibChoose",
										  RepeatDirection = RepeatDirection.Horizontal,
										  CssClass = CssClass,
										  RepeatLayout = RepeatLayout.Flow
									  };
				_LibChoose.Items.Add(new ListItem(Helper.GetResource("Document"), "Document") { Enabled = _DocLib != null });
				_LibChoose.Items.Add(new ListItem(Helper.GetResource("Picture"), "Picture") { Enabled = _PicLib != null });
				foreach (ListItem item in _LibChoose.Items)
				{
					// select the first item, which is enabled
					if (!item.Enabled) continue;
					item.Selected = true;
					break;
				}
				Controls.Add(_LibChoose);
				Controls.Add(new LiteralControl("&nbsp;"));

				var button = new Button { ID = "UploadButton", Text = Helper.GetResource("Upload") };
				if (_LibChoose.SelectedIndex == -1) button.Enabled = false;
				button.Click += UploadButtonClick;
				Controls.Add(button);
			}
			catch (Exception ex)
			{
				HandleException(string.Format("Error in CreateChildControls: {0}<br/>", ex));
			}
		}

		void UploadButtonClick(object sender, EventArgs e)
		{
			try
			{
				if (_FileUpload.HasFile == false) return;
				SPWeb web = SPContext.Current.Web;
				SPList list = _LibChoose.SelectedValue == "Document" ? _DocLib : _PicLib;
				Helper.WriteUlsVerbose(string.Format("Selected library is '{0}'", list.Title));
				UploadFile(web, _FileUpload, list);
				InsertLink(web, _FileUpload.FileName, list);
			}
			catch (Exception ex)
			{
				HandleException(string.Format("Error in UploadButtonClick: {0}", ex));
			}
		}

		public override void UpdateFieldValueInItem()
		{
			EnsureChildControls();

			try
			{
				// upload file, and insert link
			}
			catch (Exception ex)
			{
				ErrorMessage += string.Format("Error in UpdateFieldValueInItem: {0}<br/>", ex);
			}
		}

		private void GetLibraries()
		{
			try
			{
			   Guid doclibId;
				if (field.DocumentLibraryName.TryGuidParse(out doclibId))
               _DocLib = Web.Lists.GetList(doclibId, false);

			   Guid pictureLibId;
				if (field.PictureLibraryName.TryGuidParse(out pictureLibId))
               _PicLib = Web.Lists.GetList(pictureLibId, false);

				if (Fields.ContainsField(field.FieldToInsertLink))
					_FieldToInsertLink = Fields.GetFieldByInternalName(field.FieldToInsertLink);
			}
			catch (Exception ex)
			{
				HandleException(ex.Message);
			}
		}

		private void HandleException(string message)
		{
			Helper.WriteUlsError(string.Format("Error: {0}", message));
			Controls.Add(new Label { CssClass = "ms-error", Text = string.Format("{0}<br/>", message) });
		}

		private void UploadFile(SPWeb web, FileUpload fileUpload, SPList list)
		{
			try
			{
				Helper.WriteUlsVerbose(string.Format("Uploading file to list '{0}'", list.Title));
				SPFolder folder = list.RootFolder;
				SPFileCollection files = folder.Files;

				SPFile file;
				string fileUrl = string.Empty;
				try
				{
					fileUrl = string.Format("{0}/{1}/{2}", web.ServerRelativeUrl, folder.Url, fileUpload.FileName);
					// remove trailing // if we got the rootWeb
					if (fileUrl.StartsWith("//")) fileUrl = fileUrl.Substring(1);
					Helper.WriteUlsVerbose("FileUrl is " + fileUrl);
					file = web.GetFile(fileUrl);
				}
				catch (ArgumentException ex)
				{
					Helper.WriteUlsError(string.Format("Error getting file from url '{0}': {1}", fileUrl, ex));
					return;
				}

				if (!file.Exists)
				{
					var fileNameWithPath = string.Format("{0}/{1}", folder.Url, fileUpload.FileName);
					Helper.WriteUlsVerbose("Uploading file to " + fileNameWithPath);
					files.Add(fileNameWithPath, fileUpload.FileBytes);
					Controls.AddLinebreak(1);
					Controls.Add(new Label { Text = Helper.GetResource("UploadCompleted"), CssClass = CssClass });
				}
				else
				{
					Controls.Add(new Label { Text = Helper.GetResource("FileExists"), CssClass = CssClass });
				}
			}
			catch (Exception ex)
			{
				HandleException(string.Format("Error in UploadFile: {0}", ex));
			}
		}

		/// <summary>
		/// update wiki entry with url to file/image
		/// </summary>
		/// <param name="web"></param>
		/// <param name="filename"></param>
		/// <param name="library"></param>
		private void InsertLink(SPWeb web, string filename, SPList library)
		{
			try
			{
				string fieldTitle = _FieldToInsertLink.Title;
				Helper.WriteUlsVerbose(string.Format("Inserting link into field '{0}'", fieldTitle));
				string path = string.Format("{0}/{1}/{2}", web.ServerRelativeUrl != "/" ? web.ServerRelativeUrl : string.Empty, library.RootFolder.Url, filename);
				string urlToInsert = library.BaseTemplate == SPListTemplateType.PictureLibrary ?
					 string.Format("<img src=\"{0}\" border=\"0\" alt=\"{1}\">", path, filename) :
					 string.Format("<a href=\"{0}\">{1}</a>", path, filename);
				Helper.WriteUlsVerbose("UrlToInser: " + urlToInsert);

				// create javascript
				// if the existing text contains a <HR>, it is most likely a reply. so add the picture at the beginning instead of the end
            var js = new StringBuilder("\r\n");
				js.Append(Helper.JavaScriptGetTagFromIdentifierAndTitle());
				js.AppendFormat("var field = getTagFromIdentifierAndTitle('textarea','TextField','{0}');\r\n", fieldTitle);
				js.Append("if (field.value.length > 6)\r\n");
				js.Append("{\r\n");
			   SPList list = SPContext.Current.List;
			   if (list != null && list.BaseTemplate == SPListTemplateType.DiscussionBoard)
				{
					js.Append("     if (field.value.contains('<HR>'))\r\n");
					js.Append("     {\r\n");
					js.Append("         field.value = field.value.substr(5);\r\n");
               js.AppendFormat("         field.value = '<DIV>' + '{0}' + field.value;\r\n", urlToInsert);
					js.Append("     }\r\n");
					js.Append("     else\r\n");
					js.Append("     {\r\n");
               js.AppendFormat("         field.value = field.value.substr(0, field.value.length-6) + '{0}' + '</DIV>';\r\n", urlToInsert);
					js.Append("     }\r\n");
				}
				else
				{
					js.AppendFormat("     field.value = field.value.substr(0, field.value.length-6) + '{0}' + '</DIV>';\r\n", urlToInsert);
				}
				js.Append("}\r\n");
				js.Append("else if (field.value.length == 0)\r\n");
				js.Append("{\r\n");
				js.AppendFormat("     field.value = '<DIV>' + '{0}' + '</DIV>';\r\n", urlToInsert);
            js.Append("}\r\n");

				// add javascript to page
				Helper.WriteUlsVerbose("Registering JavaScript");
				ScriptManager.RegisterStartupScript(this, typeof(RHUploadAndInsert), "setValue", js.ToString(), true);
			}
			catch (Exception ex)
			{
				HandleException(string.Format("Error in InsertLink: {0}", ex));
			}
		}
	}
}