using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using RH.Update;

namespace RH.UploadAndInsertField
{
	public class RHUploadAndInsertFieldEditor : UserControl, IFieldEditor
	{
		private RHUploadAndInsert _FldRhUploadAndInsert;
		protected Label labelDescription;
		protected Label labelDocLib;
		protected Label labelPicLib;
		protected Label labelFieldToInsert;
		protected Label labelError;
		protected DropDownList ddlDocLib;
		protected DropDownList ddlPicLib;
		protected DropDownList ddlFieldToInsert;
		protected Label CurrentVersion;
		protected Label ServerVersion;
		private readonly Version _CurrentVersion = new Version(1, 0, 6, 0);
		private bool _Debug;
		private SPFieldCollection _Fields;

		public void InitializeWithField(SPField field)
		{
			try
			{
				_FldRhUploadAndInsert = (RHUploadAndInsert)field;

				if (Page.IsPostBack) return;

				_Debug = Page.GetQueryString("Debug") == "TRUE";
				_Fields = SPContext.Current.List != null ? SPContext.Current.List.Fields : SPContext.Current.Web.Fields;

				FillDropDown(ddlDocLib, SPListTemplateType.DocumentLibrary);
				FillDropDown(ddlPicLib, SPListTemplateType.PictureLibrary);
				FillFields();
				labelDescription.Text = Helper.GetResource("labelFieldDescription");
				labelDocLib.Text = Helper.GetResource("labelDocLib");
				labelPicLib.Text = Helper.GetResource("labelPicLib");
				labelFieldToInsert.Text = Helper.GetResource("labelFieldToInsert");

				if (field != null)
				{
					WriteULSVerbose("Field != null. Setting values.");
					// set values from field properties
					ddlDocLib.SelectedValue = _FldRhUploadAndInsert.DocumentLibraryName;
					ddlPicLib.SelectedValue = _FldRhUploadAndInsert.PictureLibraryName;
					ddlFieldToInsert.SelectedValue = _FldRhUploadAndInsert.FieldToInsertLink;
				}
				else
				{
					WriteULSVerbose("Field == null. Setting default values.");
					// set default properties
					SetDefaultValues();
				}
				CurrentVersion.Text = _CurrentVersion.ToString();
			}
			catch (Exception ex)
			{
				string error = string.Format("Error in InitializeWithField: {0}<br/>", ex);
				labelError.Text += error;
				Helper.WriteUlsError(error);
			}
		}

		private void SetDefaultValues()
		{
			foreach (SPList docLib in SPContext.Current.Web.Lists)
			{
				if (docLib.BaseTemplate != SPListTemplateType.DocumentLibrary) continue;
				ddlDocLib.SelectedValue = docLib.ID.ToString();
				break;
			}
			foreach (SPList picLib in SPContext.Current.Web.Lists)
			{
				if (picLib.BaseTemplate != SPListTemplateType.PictureLibrary) continue;
				ddlPicLib.SelectedValue = picLib.ID.ToString();
				break;
			}

			var list = SPContext.Current.List;
			if (list == null) return;

			if (list.BaseTemplate == SPListTemplateType.WebPageLibrary)
			{
				foreach (SPContentType contentType in list.ContentTypes)
				{
					if (contentType.NewFormTemplateName == "WikiEditForm")
					{
						// got a wiki library
						ddlFieldToInsert.SelectedValue = list.Fields[SPBuiltInFieldId.WikiField].InternalName;
					}
				}
			}

			if (list.BaseTemplate == SPListTemplateType.DiscussionBoard)
			{
				// got a discussion list
				ddlFieldToInsert.SelectedValue = list.Fields[SPBuiltInFieldId.Body].Id.ToString();
			}
		}

		/// <summary>
		/// Fill Dropdown with lists from this type
		/// </summary>
		/// <param name="dropDownList"></param>
		/// <param name="listType"></param>
		private void FillDropDown(ListControl dropDownList, SPListTemplateType listType)
		{
			WriteULSVerbose("FillDropDown: (Control: " + dropDownList.ID + ")");
			dropDownList.Items.Clear();
			foreach (SPList list in SPContext.Current.Web.Lists)
			{
				WriteULSVerbose("FillDropDown: List " + list.Title);
				if (list.BaseTemplate != listType || list.Hidden) continue;

				WriteULSVerbose("FillDropDown: Adding list to dropdown");
				var item = new ListItem(list.Title, list.ID.ToString());
				dropDownList.Items.Add(item);
			}
		}

		/// <summary>
		/// only note fields are relevant for inserting links
		/// </summary>
		private void FillFields()
		{
			WriteULSVerbose("FillFields");
			if (ddlFieldToInsert == null || ddlFieldToInsert.Items == null)
			{
				Helper.WriteUlsError("FillFields: ddlFieldToInsert or Items is null!");
				return;
			}
			ddlFieldToInsert.Items.Clear();

			foreach (SPField field in _Fields)
			{
				WriteULSVerbose("FillFields: Field: (Title: " + field.Title + ", internalName: " + field.InternalName + ")");
				// skip fields we don't want
				if (field.Type != SPFieldType.Note || ((SPFieldMultiLineText)field).RichText == false
					 || field.ReadOnlyField || field.Hidden) continue;

				WriteULSVerbose("FillFields: Adding field to dropdown. FieldType: " + field.TypeAsString);
				ddlFieldToInsert.Items.Add(new ListItem(field.Title, field.InternalName));
			}
		}

		public void OnSaveChange(SPField field, bool bNewField)
		{
			var lookup = (RHUploadAndInsert)field;

			lookup.IsNew = bNewField;
			lookup.DocumentLibraryName = ddlDocLib.SelectedValue;

			lookup.PictureLibraryName = null;
			if (string.IsNullOrEmpty(ddlFieldToInsert.SelectedValue) == false)
			{
				_Fields = SPContext.Current.List != null ? SPContext.Current.List.Fields : SPContext.Current.Web.Fields;

				// only store the image library, if the field supports FullHtml
				var noteField = (SPFieldMultiLineText)_Fields.GetFieldByInternalName(ddlFieldToInsert.SelectedValue);
				if (noteField.RichTextMode == SPRichTextMode.FullHtml)
				{
					lookup.PictureLibraryName = ddlPicLib.SelectedValue;
				}
			}
			lookup.FieldToInsertLink = ddlFieldToInsert.SelectedValue;
			lookup.ShowInDisplayForm = false;
			lookup.ShowInVersionHistory = false;
			lookup.ShowInViewForms = false;
		}

		// Properties
		public bool DisplayAsNewSection
		{
			get { return false; }
		}

		protected void updateButton_Click(object sender, EventArgs e)
		{
			try
			{
				var latest = new Updates("RH.UploadAndInsertField");
				LatestVersion latestVersion = latest.GetLatestVersion();
				if (latestVersion != null && latestVersion.Version != null)
				{
					ServerVersion.Text = latestVersion.Version.ToString();
				}
			}
			catch (Exception ex)
			{
				string error = string.Format("<br/>Error: {0}", ex.Message);
				labelError.Text += error;
				Helper.WriteUlsError(error);
			}
		}

		/// <summary>
		/// write to the ULS Log (TraceSeverity = Verbose)
		/// </summary>
		/// <param name="text"></param>
		internal void WriteULSVerbose(string text)
		{
			try
			{
				if (!_Debug) return;

				TraceProvider.RegisterTraceProvider();
				TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Verbose, Guid.Empty, "RH.UploadAndInsertField.dll", "RH.UploadAndInsertField", "General", text);
				TraceProvider.UnregisterTraceProvider();
			}
			catch (Exception)
			{
			}
		}
	}
}